@@ -3,7 +3,8 @@ import { NOTIFICATION_TYPES, SYNC_DIRECTIONS } from '@tamanu/constants';
 import { getCurrentDateTimeString } from '@tamanu/utils/dateTime';
 import { Model } from './Model';
 import { dateTimeType, type InitOptions, type Models } from '../types/model';
-import { buildSyncLookupSelect } from '../sync/buildSyncLookupSelect';
+import { EncounterPrescription } from './EncounterPrescription';
+import { buildEncounterLinkedLookupSelect } from '../sync/buildEncounterLinkedLookupFilter';
 
 export class Prescription extends Model {
   declare id: string;
@@ -34,6 +35,8 @@ export class Prescription extends Model {
   declare discontinuingClinicianId?: string;
   declare medicationId?: string;
 
+  declare encounterPrescription?: EncounterPrescription;
+
   static initModel({ primaryKey, ...options }: InitOptions, models: Models) {
     super.init(
       {
@@ -175,13 +178,15 @@ export class Prescription extends Model {
 
   static async buildSyncLookupQueryDetails() {
     return {
-      select: await buildSyncLookupSelect(this, {
+      select: await buildEncounterLinkedLookupSelect(this, {
         patientId: 'COALESCE(encounters.patient_id, patient_ongoing_prescriptions.patient_id)',
       }),
       joins: `
         LEFT JOIN encounter_prescriptions ON prescriptions.id = encounter_prescriptions.prescription_id
         LEFT JOIN encounters ON encounter_prescriptions.encounter_id = encounters.id
         LEFT JOIN patient_ongoing_prescriptions ON prescriptions.id = patient_ongoing_prescriptions.prescription_id
+        LEFT JOIN locations ON encounters.location_id = locations.id
+        LEFT JOIN facilities ON locations.facility_id = facilities.id
       `,
     };
   }
