@@ -3,12 +3,10 @@ import config from 'config';
 import { SendStatusToMetaServer } from '@tamanu/shared/tasks/SendStatusToMetaServer';
 
 // import { SenaitePoller } from './SenaitePoller';
-import { MedicationDiscontinuer } from './MedicationDiscontinuer';
 import { RefreshUpcomingVaccinations } from './RefreshMaterializedView';
 import { TimeSyncTask } from './TimeSyncTask';
 
 const DEFAULT_TASK_CLASSES = [
-  MedicationDiscontinuer,
   RefreshUpcomingVaccinations,
   SendStatusToMetaServer,
   TimeSyncTask,
